
package CDToc;

use strict;
use vars qw( @ISA @EXPORT );
use Fcntl;

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw (fetch_toc);

## This only works on Linux... 
require 'linux/cdrom.ph';

# See /usr/src/linux/include/linux/cdrom.h for details.
## struct cdrom_tochdr     
##{
##        u_char  cdth_trk0;      /* start track */
##        u_char  cdth_trk1;      /* end track */
##};
my $tochdr_t="CC";

#struct cdrom_tocentry 
#{
#        u_char  cdte_track;
#        u_char  cdte_adr        :4;
#        u_char  cdte_ctrl       :4;
#        u_char  cdte_format;
#        union cdrom_addr cdte_addr;
#        u_char  cdte_datamode;
#};
my $tocentry_t="CCCCCCC CCCCC";

## in that mess, a cdrom_addr is really:
#struct cdrom_msf0               
#{
#    u_char  minute;
#    u_char  second;
#    u_char  frame;
#};
						

sub fetch_toc ($) {
	my $dev_file = shift || die $!;

	my @toc = ();
	
	sysopen(CD, "$dev_file",O_NONBLOCK) || die "Couldn't open $dev_file: $!";

	my $header="\0\0";
				
	ioctl(CD, &CDROMREADTOCHDR, $header) || die("Read TOCHDR: $!");
	my ($start_tr,$end_tr) = unpack($tochdr_t, $header);

	my $i;
	for($i=$start_tr;$i<=$end_tr;$i++) {
		my $tocentry = pack($tocentry_t, $i, 0, &CDROM_MSF, 0, 0, 0, 0, 0);
	
		ioctl(CD, &CDROMREADTOCENTRY, $tocentry) || die("Read TOC: $!");
		my ($track,$adr,$ctrl,$format,$min,$sec,$fr,$datamode) = unpack($tocentry_t, $tocentry);
		#print "Track $i: $min min $sec sec $fr frames\n";
		push(@toc, "$i $min $sec $fr");	
	}

	## Fetch the leadout info
	my $tocentry = pack($tocentry_t, &CDROM_LEADOUT, 0, &CDROM_MSF, 0, 0, 0, 0);
	ioctl(CD, &CDROMREADTOCENTRY, $tocentry) || die("Fetching leadout: $!");
	my ($track,$adr,$ctrl,$format,$min,$sec,$fr,$datamode) = unpack($tocentry_t, $tocentry);
	push(@toc, "999  $min $sec $fr");	
	#print "Leadout: $min min $sec sec $fr frames\n";
	close(CD);

	return @toc;
}

1;
							 
